# Copyright 2018 Yanis Guenane  <yanis@guenane.org>
# Author: Yanis Guenane  <yanis@guenane.org>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import flask
import json
import logging
import os
import threading
import uuid

from dcifeeder import routine
from dcifeeder import settings


def dump_settings(app):
    symbols = filter(lambda x: x.isupper(), dir(settings))
    values = u'\n\t'.join(
        (u"%s: '%s'" % (sym,
                        getattr(settings, sym) if 'PASSWORD' not in sym
                        else '*** REDACTED ***')
         for sym in symbols))
    app.logger.info(u"Dumping settings:\n\t%s" % values)


app = flask.Flask(__name__)
dump_settings(app)


def configure_logger(event_id):
    logger = logging.getLogger()
    logger.setLevel(logging.INFO)

    for handler in logger.handlers[:]:
        logger.removeHandler(handler)

    formatter = logging.Formatter(
        '%(asctime)s,%(msecs)d %(name)s %(levelname)s %(message)s'
    )
    handler = logging.FileHandler(
        os.path.join(settings.LOG_DIR, event_id), mode='a'
    )
    handler.setLevel(logging.INFO)
    handler.setFormatter(formatter)

    logger.addHandler(handler)


@app.route('/', strict_slashes=False)
def index():
    return flask.Response(
        json.dumps({'_status': 'OK',
                    'message': 'Distributed CI Feeder as a Service. GET /options to see available options'}),  # noqa
        status=200,
        content_type='application/json'
    )


@app.route('/options', strict_slashes=False)
def options():
    return flask.Response(
        json.dumps(
            {
                'message': 'Available call POST /product/topic (ie. POST /openstack/osp8)',  # noqa
                'product': {
                    'openstack': ['osp8', 'osp9', 'osp10', 'osp11', 'osp12',
                                  'osp13', 'osp14', 'rdonewton', 'rdoocata',
                                  'rdopike', 'rdoqueens', 'rdopike',
                                  'rdorocky'],
                    'rhel': ['rhel7', 'rhel8'],
                    'ansible': ['ansible26'],
                    'openshift': ['ocp3_10', 'ocp3_11', 'ocp4_0']
                }
            }
        ),
        status=200,
        content_type='application/json'
    )


@app.route('/<product>/<topic>', strict_slashes=False, methods=['POST'])
def kick(product, topic):
    if not routine.get_topic_class(product, topic):
        return flask.Response(
            json.dumps({'message': 'Pair %s/%s does not exists. Please query GET /options.' % (product, topic)}),  # noqa
            status=404, content_type='application/json'
        )

    run_file = os.path.join(
        settings.CACHE_DIR,
        '.' + routine.get_topic_class(product, topic)().name
    )
    if os.path.exists(run_file):
        with open(run_file) as run_file_fd:
            return flask.Response(
                json.dumps({
                    'message':
                        'An import for this topic is already in progress.',
                    'event_id': run_file_fd.read()
                }),
                status=302, content_type='application/json'
            )

    event_id = str(uuid.uuid4())
    configure_logger(event_id)
    thread = threading.Thread(
        target=routine.start_feeder, args=(product, topic)
    )
    thread.start()
    return flask.Response(
        json.dumps({'_status': 'OK',
                    'message': 'Feeder for %s/%s has been started.' % (product,
                                                                       topic),
                    'event_id': event_id}),
        status=201,
        content_type='application/json'
    )


@app.route('/logs/<event_id>', strict_slashes=False)
def get_job_log(event_id):
    try:
        with open(os.path.join(settings.LOG_DIR, str(event_id))) as log_fd:
            return flask.Response(
                log_fd.read(),
                content_type='text/plain'
            )
    except Exception:
        return flask.Response(None, status=404)
