# Copyright 2018 Yanis Guenane  <yanis@guenane.org>
# Author: Yanis Guenane  <yanis@guenane.org>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import StringIO
import logging
import os
import requests
import shutil
import tarfile
import yaml

from dcifeeder.feeder import run_command, build_dir, get_topic_from_name
from dcifeeder import settings

try:
    import ConfigParser as configparser
except ImportError:
    import configparser

logger = logging.getLogger(__name__)


class BaseComponent(object):

    def __init__(self, metadata_url, archive_name='archive.tar'):
        self.metadata_url = metadata_url
        self.archive_name = archive_name
        self.state = 'active'
        self.retrieve_metadata(self.metadata_url)
        self.base_url = '/'.join(self.metadata_url.split('/')[:-1])
        self.api_accepted_fields = ['title', 'message', 'data', 'name',
                                    'canonical_project_name', 'export_control',
                                    'url', 'type', 'state']

    def serialize(self):
        component = dict(self)
        for elem in ['name', 'type']:
            del component[elem]

        return component

    def __str__(self):
        component = {}
        fields = self.api_accepted_fields + ['id', 'etag']
        for field in fields:
            if hasattr(self, field):
                component[field] = getattr(self, field)

        return str(component)

    def __iter__(self):
        for field in self.api_accepted_fields:
            if hasattr(self, field):
                yield(field, getattr(self, field))

    def is_healthy(self):
        return self.is_status_finished()

    def is_status_finished(self):
        url = '%s/STATUS' % self.base_url
        r = requests.get(url)
        if r.status_code != 200 or r.text.rstrip() != 'FINISHED':
            logger.error('%s: Missing or file content not FINISHED' % url)
            return False
        return True

    def build_archive(self):
        logger.info('Running: build_archive')
        if os.path.exists(self.archive_name):
            os.remove(self.archive_name)
        with tarfile.open(self.archive_name, 'w:') as tar:
            tar.add(self.canonical_project_name)
        logger.info('... build_archive done.')

    def sync(self):
        raise NotImplementedError

    def retrieve_metadata(self, metadata_url):
        raise NotImplementedError


class ReposyncComponent(BaseComponent):

    def __init__(self, metadata_url, archive_name='archive.tar'):
        super(ReposyncComponent, self).__init__(metadata_url,
                                                archive_name)

    @staticmethod
    def parse_repo_file(repo_file):
        output = StringIO.StringIO(repo_file)
        config = configparser.ConfigParser()
        config.readfp(output)

        section_name = config.sections()[0]
        repo_name = config.get(section_name, 'name')
        base_url = (config.get(section_name, 'baseurl')
                    .replace("$basearch", "x86_64"))

        return {
            'base_url': base_url,
            'name': repo_name,
        }

    @staticmethod
    def parse_dlrn_informations(dlrn_data):
        commit_information = yaml.load(dlrn_data)['commits'][0]

        return {
            'commit_hash': commit_information.get('commit_hash'),
            'distro_hash': commit_information.get('distro_hash'),
            'commit_branch': commit_information.get('commit_branch')
        }

    def sync(self):

        yum_config = '[{}]\nname = {}\nbaseurl = {}\nenabled = 1' \
            .format(self.canonical_project_name, self.canonical_project_name,
                    self.url)
        with open('yum.conf', 'w') as yum_fd:
            yum_fd.write(yum_config)
        logger.info('Syncing content ...')
        logger.info(yum_config)

        reposync_command = ["reposync", "-c", "yum.conf",
                            "--download-metadata", "--norepopath", "-r",
                            self.canonical_project_name, "-p",
                            self.canonical_project_name]
        run_command(reposync_command)
        logger.info('... done syncing.')

        logger.info('Running createrepo ...')
        run_command(["createrepo", self.canonical_project_name])
        logger.info('... createrepo done.')

        topic = get_topic_from_name(self.topic_name)
        base_path = os.path.join(
            settings.CACHE_DIR,
            topic['product']['label'].lower(),
            topic['id']
        )
        logger.info('Copying the data to component id from %s to %s...' %
                    (os.path.abspath(self.canonical_project_name),
                     os.path.join(base_path, self.id)))
        build_dir(base_path)
        shutil.copytree(
            self.canonical_project_name,
            os.path.join(base_path, self.id)
        )
        logger.info('... done copying the data.')


class RsyncComponent(BaseComponent):

    def __init__(self, metadata_url, archive_name='archive.tar'):
        super(RsyncComponent, self).__init__(metadata_url, archive_name)

    def retrieve_metadata(self, url):
        r = requests.get(url)

        self.name = r.text
        self.canonical_project_name = r.text
        baseurl = '/'.join(url.split('/')[:-2])
        self.url = '/'.join((baseurl, r.text))

    @property
    def rsync_host(self):
        return self.url.split('/')[2]

    @property
    def rsync_source_path(self):
        raise NotImplementedError()

    @property
    def rsync_destination_path(self):
        raise NotImplementedError()

    @property
    def component_path(self):
        return os.path.abspath(os.path.join(
            self.rsync_destination_path,
            '..',
            self.id))

    def sync(self):
        build_dir(self.rsync_destination_path)
        build_dir(os.path.abspath(self.canonical_project_name))

        rsync_src = "%s::%s" % (self.rsync_host, self.rsync_source_path)
        commands = [
            ["rsync", "-av", rsync_src, self.rsync_destination_path],
            ["cp", "-al", self.rsync_destination_path, self.component_path]
        ]

        for command in commands:
            run_command(command)
