# Copyright 2018 Yanis Guenane  <yanis@guenane.org>
# Author: Yanis Guenane  <yanis@guenane.org>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import dciclient.v1.api.component as dci_component
import dciclient.v1.api.context as dci_context
import dciclient.v1.api.topic as dci_topic
import errno
import logging
import os

from dcifeeder import settings
from dcifeeder.exceptions import FailureException
from functools import wraps
from subprocess import check_output, STDOUT, CalledProcessError

logger = logging.getLogger(__name__)


def contextify(f):
    @wraps(f)
    def func(*args, **kwargs):
        context = dci_context.build_dci_context(
            settings.DCI_CS_URL, settings.DCI_LOGIN,
            settings.DCI_PASSWORD
        )
        return f(context, *args, **kwargs)

    return func


def run_command(command):
    command_str = " ".join(command)
    logger.debug("Running command: '%s'" % command_str)
    try:
        stdout = check_output(command, stderr=STDOUT)
    except CalledProcessError as e:
        logger.error("Command '%s' exited with return code: %d" %
                     (command_str, e.returncode))
        logger.error("Output was: %s" % e.output)
        raise FailureException("An error occured while running: %s. "
                               "Return code: %d" %
                               (command, e.returncode))

    logger.debug("Command `%s` succeeded." % command_str)
    logger.debug("Output was: %s" % stdout)
    return stdout


def build_dir(path):

    try:
        os.makedirs(path)
    except OSError as exc:
        if exc.errno == errno.EEXIST and os.path.isdir(path):
            pass
        else:
            raise FailureException(
                'An error occured while creating dir: %s' % path
            )


@contextify
def get_topic_from_name(context, name):
    r = dci_topic.list(context, where='name:' + name, limit=1, embed='product')
    return r.json()['topics'][0]


@contextify
def get_or_create_component(context, topic, component):
    logger.info("Get or create component: {}".format(component.name))

    r = dci_component.create(
        context, component.name, component.type, topic.id,
        **component.serialize()
    )
    created = False
    if r.status_code == 201:
        created = True
        created_component = r.json()['component']
        setattr(component, 'id', created_component['id'])
        setattr(component, 'etag', created_component['etag'])
    return component, created


@contextify
def upload_component(context, workspace, component):
    logger.info('Uploading content for component {}'.format(component.name))

    archive_path = os.path.join(workspace, component.archive_name)
    if not os.path.exists(archive_path):
        return

    r = dci_component.file_upload(
        context, component.id, file_path=archive_path
    )

    if r.status_code == 201:
        logger.info(
            'Component succesfully uploaded: {}'.format(component.name)
        )
    else:
        msg = "Component upload failed: {}".format(component.name)
        raise FailureException(msg)


@contextify
def update_component(context, component, data):
    data['etag'] = component.etag
    logger.info("Update component: {}\n     Data: {}".format(component.name,
                                                             data))

    r = dci_component.update(context, component.id, **data)
    if r.status_code == 200:
        logger.info('Component succesfully updated: {}'.format(component.name))
        updated_component = r.json()['component']
        setattr(component, 'etag', updated_component['etag'])
    else:
        msg = "Component update failed: {}. Status code {}: {}".format(
            component.name, r.status_code, r.text)
        raise FailureException(msg)
