# Copyright 2018 Yanis Guenane  <yanis@guenane.org>
# Author: Yanis Guenane  <yanis@guenane.org>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging
import requests

from dcifeeder.component import ReposyncComponent
from six.moves.urllib.parse import urlparse

logger = logging.getLogger(__name__)


class BaseOSPComponent(ReposyncComponent):

    def __init__(self, metadata_url, archive_name='archive.tar'):
        super(BaseOSPComponent, self).__init__(metadata_url,
                                               archive_name)
        self.topic_name = 'OSP%s' % self.name.split('-')[2].split('.')[0]
        self.healthy = (
            self.is_not_under_embargo() and
            self.is_status_file_correct()
        )

    def retrieve_metadata(self, url):
        r = requests.get(url)

        repo = self.parse_repo_file(r.text)
        version = repo['base_url'].split('/')[-4]
        self.canonical_project_name = repo['name']
        self.name = '%s %s' % (repo['name'], version)
        self.url = repo['base_url']
        self.data = {
            'path': urlparse(repo['base_url']).path,
            'version': version,
            'repo_name': repo['name'],
        }

    def is_healthy(self):
        return self.healthy

    def is_not_under_embargo(self):
        r = requests.get('%s/EMBARGOED' % self.base_url)
        if r.status_code == 200:
            logger.error('EMBARGOED: Puddle is under embargo')
            return False
        return True

    def is_status_file_correct(self):
        r = requests.get('%s/status.txt' % self.base_url)
        if r.status_code != 200 or r.text.rstrip() != 'COMPLETE':
            logger.error('status.txt: Missing or file content not COMPLETE')
            return False
        return True

    def is_import_data_present(self):
        r = requests.get('%s/import_data/commit.yaml' % self.base_url)
        if r.status_code != 200:
            logger.error('import_data/commit.yaml: Missing file')
            return False
        return True

    def is_overcloud_container_present(self):
        r = requests.get(
            '%s/overcloud_container_image_prepare.yaml' % self.base_url
        )
        if r.status_code != 200:
            logger.error(
                'overcloud_container_image_prepare.yaml: Missing file'
            )
            return False
        return True


class OSPComponent(BaseOSPComponent):

    def __init__(self, metadata_url, archive_name='archive.tar'):
        super(OSPComponent, self).__init__(metadata_url, archive_name)
        self.type = 'puddle_osp'


class OSPDComponent(BaseOSPComponent):

    def __init__(self, metadata_url, archive_name='archive.tar'):
        super(OSPDComponent, self).__init__(metadata_url, archive_name)
        self.type = 'puddle_ospd'
