# Copyright 2018 Yanis Guenane  <yanis@guenane.org>
# Author: Yanis Guenane  <yanis@guenane.org>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import docker
import logging
import openshift.client
import requests
import traceback
import yaml

from dcifeeder import settings
from dcifeeder.component import BaseComponent
from dcifeeder.exceptions import FailureException
from dcifeeder.products.openstack.components.osp import OSPComponent
from dcifeeder.products.openstack.components.osp import OSPDComponent
from dcifeeder.topic import BaseTopic

logger = logging.getLogger(__name__)
client = docker.from_env()


class OSP(BaseTopic):

    def __init__(self):
        self.name = self.__class__.__name__
        super(OSP, self).__init__()
        self.version = '%s.0' % self.name[3:]
        self.url = \
            'http://download.eng.bos.redhat.com/rcm-guest/puddles/\
OpenStack/%s-RHEL-7/passed_phase1/RH7-RHOS-%s.repo' % \
            (self.version, self.version)
        self.components = [OSPComponent(self.url)]


class OSPWithDirector(OSP):

    def __init__(self):
        super(OSPWithDirector, self).__init__()
        self.url_director = \
            'http://download.eng.bos.redhat.com/rcm-guest/puddles/\
OpenStack/%s-RHEL-7-director/passed_phase1/RH7-RHOS-%s-director.repo' % \
            (self.version, self.version)
        self.components.append(OSPDComponent(self.url_director))


class OSPWithContainers(OSP):

    def __init__(self):
        super(OSPWithContainers, self).__init__()
        dlrn_url = \
            '%s/import_data/commit.yaml' % '/'.join(self.url.split('/')[:-1])
        component = self.components[0]

        component.data['dlrn'] = \
            component.parse_dlrn_informations(requests.get(dlrn_url).text)

        component.healthy = (
            component.healthy and
            component.is_import_data_present() and
            component.is_overcloud_container_present()
        )

    def _sync_images(self, overcloud_container_image_prepare):

        def parse_output(lines):
            for line in lines:
                if 'errorDetail' in line:
                    raise FailureException(line[:-1])

        def registry_login(username, password, registry):
            logger.info('login onto %s' % registry)
            try:
                client.login(
                    username=username, password=password, email='',
                    registry=registry, reauth=True
                )
            except Exception:
                raise FailureException(traceback.format_exc())

        def pull_image(image):
            if image['origin_project']:
                format_str = '{origin_registry}/{origin_project}/{name}'.format(**image)  # noqa
            else:
                format_str = '{origin_registry}/{name}'.format(**image)

            logger.info('docker pull %s' % format_str)

            result = client.pull(format_str, tag=image['origin_tag'], stream=True)
            parse_output(result)

        def push_image(image):
            logger.info('docker push {dest_registry}/{project}/{name}:{tag}'.format(**image))  # noqa

            result = client.push('{dest_registry}/{project}/{name}'.format(**image), tag=image['tag'], stream=True)  # noqa
            parse_output(result)

        def tag_image(image):
            logger.info('docker tag {name} {dest_registry}/{project}/{name}:{tag}'.format(**image))  # noqa

            if image['origin_project']:
                format_str = '{origin_registry}/{origin_project}/{name}:{origin_tag}'.format(**image)  # noqa
            else:
                format_str = '{origin_registry}/{name}:{origin_tag}'.format(**image)  # noqa

            try:
                docker_image_id = client.inspect_image(format_str)['Id']  # noqa
                client.tag(docker_image_id, repository='{dest_registry}/{project}/{name}'.format(**image), tag=image['tag'])  # noqa
            except Exception:
                raise FailureException(traceback.format_exc())

        def list_existing_pushed_tags(image, username, password):
            url = 'https://%s/v2/%s/%s/tags/list' % (image['dest_registry'],
                                                     image['project'],
                                                     image['name'])
            payload = {'service': 'Docker registry',
                       'scope': 'repository:%s/%s:pull' % (image['project'],
                                                           image['name'])}
            auth = requests.get('https://%s/auth' % image['dest_registry'],
                                auth=(username, password), params=payload)
            if auth.status_code == 200 and auth.json().get('token'):
                token = auth.json()['token']
                headers = {'Authorization': 'Bearer %s' % token}
                r = requests.get(url, headers=headers)
                if r.status_code == 404:
                    return []
                elif r.status_code == 200 and r.json() and 'tags' in r.json():
                    return r.json()['tags']
                else:
                    raise Exception(r.text)
            else:
                raise Exception(auth.text)

        def retrieve_images_list(project):
            configuration = openshift.client.Configuration()
            configuration.username = settings.OPENSHIFT_USER
            configuration.password = settings.OPENSHIFT_PASSWORD
            configuration.host = settings.OPENSHIFT_URL
            configuration.verify_ssl = False

            try:
                api_instance = openshift.client.ImageOpenshiftIoV1Api(
                    openshift.client.ApiClient(configuration)
                )
                api_response = api_instance.list_namespaced_image_stream(
                    project
                )
            except Exception:
                raise FailureException(traceback.format_exc())

            return [i.metadata.self_link.split('/')[-1] for i in api_response.items]  # noqa

        registry_login(
            settings.DEST_REGISTRY_USER,
            settings.DEST_REGISTRY_PASSWORD,
            settings.DEST_REGISTRY_URL
        )

        container_image_prepare = \
            yaml.load(overcloud_container_image_prepare)['container-image-prepare']  # noqa
        osp_registry, osp_project = \
            container_image_prepare['namespace'].split('/')
        tag = container_image_prepare['tag']

        try:
            ceph_registry, ceph_project = \
                container_image_prepare['ceph-namespace'].split('/')
        except Exception:
            ceph_registry = container_image_prepare['ceph-namespace']
            ceph_project = None

        images = retrieve_images_list(osp_project)
        uploaded_images = []
        for image in images:
            image = {
                'name': image,
                'origin_tag': tag,
                'tag': tag,
                'origin_project': osp_project,
                'project': osp_project,
                'origin_registry': osp_registry,
                'dest_registry': settings.DEST_REGISTRY_URL
            }

            uploaded_images.append('{dest_registry}/{project}/{name}:{tag}'.format(**image))  # noqa
            if tag in list_existing_pushed_tags(image, settings.DEST_REGISTRY_USER, settings.DEST_REGISTRY_PASSWORD):  # noqa
                logger.info('{name}:{tag} already pushed'.format(**image))
                continue
            pull_image(image)
            tag_image(image)
            push_image(image)

        ceph_image = {
            'name': container_image_prepare['ceph-image'],
            'origin_tag': container_image_prepare['ceph-tag'],
            'tag': tag,
            'origin_project': ceph_project,
            'project': osp_project,
            'origin_registry': ceph_registry,
            'dest_registry': settings.DEST_REGISTRY_URL
        }
        uploaded_images.append('{dest_registry}/{project}/{name}:{tag}'.format(**ceph_image))  # noqa

        if tag in list_existing_pushed_tags(ceph_image, settings.DEST_REGISTRY_USER, settings.DEST_REGISTRY_PASSWORD):  # noqa
            logger.info('{name}:{tag} already pushed'.format(**ceph_image))
        else:
            pull_image(ceph_image)
            tag_image(ceph_image)
            push_image(ceph_image)

        return uploaded_images

    def post_component_sync(self, component):
        r = requests.get(
            self.url + '/../overcloud_container_image_prepare.yaml'
        )

        images_list = self._sync_images(r.text)
        with open(
            '%s/images_list.yaml' % component.canonical_project_name, 'w'
        ) as f:
            f.write(yaml.dump(images_list, explicit_start=True,
                              default_flow_style=False))
        BaseTopic.post_component_sync(self, component)


class OSP8(OSPWithDirector):

    def __init__(self):
        super(OSP8, self).__init__()


class OSP9(OSPWithDirector):

    def __init__(self):
        super(OSP9, self).__init__()


class OSP10(OSP):

    def __init__(self):
        super(OSP10, self).__init__()


class OSP11(OSP):

    def __init__(self):
        super(OSP11, self).__init__()
        dlrn_url = '%s/import_data/commit.yaml' % '/'.join(self.url.split('/')[:-1])  # noqa
        self.components[0].data['dlrn'] = BaseComponent.parse_dlrn_informations(requests.get(dlrn_url).text)  # noqa

        component = self.components[0]
        component.healthy = (
            component.healthy and
            component.is_import_data_present()
        )


class OSP12(OSPWithContainers):

    def __init__(self):
        super(OSP12, self).__init__()


class OSP13(OSPWithContainers):

    def __init__(self):
        super(OSP13, self).__init__()


class OSP14(OSPWithContainers):

    def __init__(self):
        super(OSP14, self).__init__()
