# Copyright 2018 Yanis Guenane  <yanis@guenane.org>
# Author: Yanis Guenane  <yanis@guenane.org>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import importlib
import logging
import os
import shutil
import traceback
import uuid

from dcifeeder import settings
from dcifeeder import feeder
from dcifeeder.exceptions import FailureException

logger = logging.getLogger(__name__)


def get_topic_class(product, topic):

    if product.lower() == 'openstack':
        topic_branch = topic[:3].lower()
        if topic_branch == 'osp':
            class_name = topic.upper()
        elif topic_branch == 'rdo':
            class_name = topic[:4].upper() + topic[4:]
    elif product.lower() == 'rhel':
        topic_branch = 'rhel'
        class_name = topic.upper()
    elif product.lower() == 'ansible':
        topic_branch = 'ansible'
        class_name = topic.capitalize()
    elif product.lower() == 'openshift':
        topic_branch = topic[:3].lower()
        class_name = topic.upper()

    try:
        module = importlib.import_module(
            'dcifeeder.products.%s.topics.%s' % (product.lower(), topic_branch)
        )
        return getattr(module, class_name)
    except Exception:
        return False


def create_and_cd_into_workspace():
    try:
        workspace = 'dci.%s' % (str(uuid.uuid4()))
        workspace = os.path.join(settings.CACHE_DIR, workspace)
        os.mkdir(workspace)
        os.chdir(workspace)
        return workspace
    except Exception:
        raise FailureException(traceback.format_exc())


def cleanup_workspace(workspace):
    logger.info('Cleaning up workspace %s' % workspace)
    if os.path.exists(workspace):
        shutil.rmtree(workspace)


def process_components(topic, workspace):
    for component in topic.components:
        logger.info(
            "Processing Component: %s - Type: %s" %
            (component.name, component.type)
        )
        component, created = feeder.get_or_create_component(topic, component)
        if not created:
            logger.info("Component already created. Skipping it.")
            continue

        feeder.update_component(component, {"state": "inactive"})
        # todo(gvincent): better feeder.sync_repos(topic, component)
        component.sync()
        topic.post_component_sync(component)
        # end todo
        # todo(gvincent): feeder.send_archive_on_control_server(workspace, component)
        component.build_archive()
        feeder.upload_component(workspace, component)
        # end todo
        feeder.update_component(
            component, {"state": "active", "export_control": True})


def start_feeder(product, topic):
    _topic = get_topic_class(product, topic)()
    workspace = create_and_cd_into_workspace()

    run_file = os.path.join(settings.CACHE_DIR, '.%s' % _topic.name)
    with open(run_file, 'w') as run_file_fd:
        log_filename = logging.getLogger().handlers[0].baseFilename.split('/')[-1]
        run_file_fd.write(log_filename)
    try:
        logger.info('Starting feeder - Workspace: %s' % workspace)
        logger.info('Product: %s' % product.upper())
        logger.info('Topic: %s' % _topic.name)
        if _topic.is_healthy():
            process_components(_topic, workspace)
    except Exception as e:
        logger.error(e.message)
        logger.error('Status: FAILURE')
    finally:
        cleanup_workspace(workspace)
        os.remove(run_file)
